package cn.rankin.task.service;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.data.api.product.vo.GoodsVo;
import cn.rankin.data.api.product.vo.ProductVo;
import cn.rankin.task.code.TaskAPICode;
import org.springframework.cloud.netflix.feign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;

@FeignClient(name = "${service.product.name}")
public interface ProductService {

    @RequestMapping(value = "/goods/ids", method = RequestMethod.GET)
    APIResult<List<GoodsVo>> findGoodsByIds(@RequestParam("id") List<String> goodsIdList);

    @RequestMapping(value = "/product/ids", method = RequestMethod.GET)
    APIResult<List<ProductVo>> findProductByIds(@RequestParam("id") List<String> pidList);

    @RequestMapping(value = "/product/{productId}", method = RequestMethod.GET)
    APIResult findProductById(@PathVariable("productId") String productId);

    @Component
    class ProductServiceHystrix implements ProductService {

        @Override
        public APIResult findGoodsByIds(List<String> goodsIdList) {
            return APIResult.error(TaskAPICode.SERVER_ERROR);
        }

        @Override
        public APIResult findProductByIds(List<String> pidList) {
            return APIResult.error(TaskAPICode.SERVER_ERROR);
        }

        @Override
        public APIResult findProductById(String productId) {
            return APIResult.error(TaskAPICode.SERVER_ERROR);
        }
    }
}
