package cn.rankin.task.service;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.enums.OrderStatusEnum;
import cn.rankin.data.api.trade.entity.UserOrder;
import cn.rankin.task.code.TaskAPICode;
import org.springframework.cloud.netflix.feign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;

@FeignClient(name = "${service.trade.name}", fallback = TradeService.TradeServiceHystrix.class)
public interface TradeService {

    @RequestMapping(value = "/order/pay/list", method = RequestMethod.GET)
    APIResult<List<UserOrder>> findPayList(@RequestParam("limit") Integer limit);

    @RequestMapping(value = "/order/pay/{orderId}", method = RequestMethod.POST)
    APIResult<Boolean> complete(@PathVariable("orderId") String orderId, @RequestParam("status") OrderStatusEnum status);

    @Component
    class TradeServiceHystrix implements TradeService {

        @Override
        public APIResult findPayList(Integer limit) {
            return APIResult.error(TaskAPICode.SERVER_ERROR);
        }

        @Override
        public APIResult complete(String orderId, OrderStatusEnum status) {
            return APIResult.error(TaskAPICode.SERVER_ERROR);
        }

    }
}

